<?php

require_once 'op5/license/op5lic.php';
require_once 'op5/license/generator.php';
require_once __DIR__.'/../check_op5_license';
require_once 'op5/sysinfo.php';

class op5sysinfo_mock extends op5sysinfo {
	public $mon;
	public $log;
	function get_monitor_usage() { return $this->mon; }
	function get_logserver_usage() { return $this->log; }
}

/**
 * This is really black box testing since it uses op5license-install et al. to install
 * fixtures.
 *
 * ### Note that your system's installed license will get destroyed. ###
 */
class LicenseTest extends PHPUnit_Framework_TestCase {

	function setup() {
		$this->command_prefix = __DIR__.'/../check_op5_license ';
		$this->license_file = tempnam('/tmp', 'check_op5_license_');
	}

	function teardown() {
		$this->install_date();
		unlink($this->license_file);
	}

	private function _query_db($query) {
		return $this->dbconn->query($query);
	}

	/**
	 * Installs test-specific license files
	 *
	 * Stolen from op5license
	 *
	 * @param $license_data array
	 * @return void
	 */
	private function mock($license_data, $version = 'op5v2') {
		$licgen = new op5license_generator($version);
		$int = new op5license_generator_interval();

		if (isset($license_data['IssuedToCustomerNumber'])) {
			$licgen->set_user_info('customer.number',
				$license_data['IssuedToCustomerNumber']);
		} else {
			$licgen->set_user_info('customer.number', '999999-01');
		}
		unset($license_data['IssuedToCustomerNumber']);
		if (isset($license_data['IssuedToName'])) {
			$licgen->set_user_info('customer.name',
				$license_data['IssuedToName']);
		} else {
			$licgen->set_user_info('customer.name', 'TestCustomer');
		}
		unset($license_data['IssuedToName']);
		if (isset($license_data['IssuedToEmail'])) {
			$licgen->set_user_info('customer.email',
				$license_data['IssuedToEmail']);
		} else {
			$licgen->set_user_info('customer.email', 'test@test.com');
		}
		unset($license_data['IssuedToEmail']);

		if (!empty($license_data['IssueDate'])) {
			if (is_numeric($license_data['IssueDate'])) {
				$int->set_start_time(intval($license_data['IssueDate']));
			} else {
				$int->set_start_time(
					strtotime($license_data['IssueDate']) + 12 * 3600);
			}
		}
		unset($license_data['IssueDate']);
		if (!empty($license_data['ExpireDate'])) {
			if (is_numeric($license_data['ExpireDate'])) {
				$int->set_end_time(intval($license_data['ExpireDate']));
			} else {
				$int->set_end_time(
					strtotime($license_data['ExpireDate']) + 12 * 3600);
			}
		}
		unset($license_data['ExpireDate']);

		foreach ($license_data as $k => $v) {
			$int->set_metric($k, $v);
		}

		$licgen->add_interval($int);
		$licgen->generate();

		$file = file_put_contents($this->license_file, $licgen->get_license());
		if ($file === false || $file < 1) {
			throw new Exception(
				'An error occured while trying to create license file', 1);
		}
	}

	/**
	 * Seeds database with report_data
	 */
	private function install_date($timestamp = 0) {
		static $dbconn;
		if(!$dbconn) {
			$dbconn = new PDO('mysql:dbname=merlin;host=127.0.0.1', 'merlin', 'merlin');
		}
		$host = 'RAMBOOOOOO';
		$service = '1stBl00od';
		if($timestamp) {
			//fwrite(STDERR, sprintf("Inserting $timestamp (%s)\n", date('Y-m-d H:i:s', $timestamp)));
			$dbconn->query("INSERT INTO report_data (timestamp, event_type, host_name, service_description, state, hard, retry) VALUES (".$timestamp.", 100, '$host', '$service', 0, 0, 0)");
		} else {
			$dbconn->query("DELETE FROM report_data WHERE host_name = '$host' AND service_description = '$service'");
		}
	}

	// @todo place this in some general helper since it's the shit for testing CLI stuff
	private function _exec($cli, $args, $expected_output, $expected_exit_code) {
		$cli_args = '';
		foreach($args as $arg => $value) {
			// treat the key as a parameter and value as argument
			// allows array('-w' => 2, '--help') to work
			if(!is_numeric($arg) && $arg[0] == '-') {
				$cli_args .= " $arg ".escapeshellarg($value);
			} else {
				$cli_args .= " ".escapeshellarg($value);
			}
		}
		$cmd = $cli.$cli_args;
		//fwrite(STDERR, $cmd."\n");
		$ph = proc_open($cmd, array(array('pipe', 'r',), array('pipe', 'w'), array('pipe', 'w')), $pipes);
		if($ph === false) {
			throw new Exception("Could not proc_open '$cmd'");
		}
		fclose($pipes[0]);
		$stdout = trim(stream_get_contents($pipes[1]));
		$stderr = trim(stream_get_contents($pipes[2]));
		$exit_code = proc_close($ph);
		if($exit_code === -1) {
			throw new Exception("Could not proc_close '$cmd'");
		}
		$this->assertEquals($exit_code, $expected_exit_code, $cmd."\nstderr:\n".$stderr."\nstdout:\n".$stdout);
		$this->assertEquals($stdout, $expected_output, $cmd."\nstderr:\n".$stderr."\nstdout:\n".$stdout);
	}

	function testStandardFlags() {
		$this->_exec($this->command_prefix, array('-V'), 'unknown', 0);
		$this->_exec($this->command_prefix, array('--version'), 'unknown', 0);
		foreach(array('-h', '--help') as $h) {
			$cmd = $this->command_prefix.$h;
			exec($cmd, $output, $exit_code);
			$output = trim(implode("\n", $output));
			$this->assertEquals($exit_code, 0, "Command '$cmd' returned '$exit_code'");
			$this->assertTrue(strpos($output, 'Show this help message') !== false, "Command '$cmd' output:\n$output");
		}
	}

	function testHostsLeft() {
		$allowed = 1000;
		$this->mock(array('monitor.hosts' => $allowed));

		$lic = new op5lic($this->license_file);
		$sys = new op5sysinfo_mock();
		$sys->mon = 10;

		$left = 990;
		$check = new Check_Op5license($lic, $sys);

		$w = 1000;
		$c = 10;
		$result = $check->check(array('w' => $w, 'c' => $c, 'T' => 'h'));
		$this->assertSame("WARNING: $left hosts left|'hosts left'=$left;$w;$c 'hosts allowed'=".$allowed, $result[1]);
		$this->assertSame(1, $result[0]);

		$w = 2000;
		$c = 1000;
		$result = $check->check(array('w' => $w, 'c' => $c, 'T' => 'h'));
		$this->assertSame("CRITICAL: $left hosts left|'hosts left'=$left;$w;$c 'hosts allowed'=".$allowed, $result[1]);
		$this->assertSame(2, $result[0]);

		$w = 50;
		$c = 20;
		$result = $check->check(array('w' => $w, 'c' => $c, 'T' => 'h'));
		$this->assertSame("OK: $left hosts left|'hosts left'=$left;$w;$c 'hosts allowed'=".$allowed, $result[1]);
		$this->assertSame(0, $result[0]);
	}

	function testDaysLeft() {
		$this->install_date(strtotime('-25 days'));
		$lic = new op5lic('/trigger/trial/please');
		$sys = new op5sysinfo_mock();
		$check = new Check_Op5license($lic, $sys);

		$left = 5;

		$w = 6;
		$c = 1;
		$result = $check->check(array('w' => $w, 'c' => $c, 'T' => 'd'));
		$this->assertSame("WARNING: $left days left|'days left'=$left;$w;$c", $result[1]);
		$this->assertSame(1, $result[0]);

		$w = 10;
		$c = 5;
		$result = $check->check(array('w' => $w, 'c' => $c, 'T' => 'd'));
		$this->assertSame("CRITICAL: $left days left|'days left'=$left;$w;$c", $result[1]);
		$this->assertSame(2, $result[0]);

		$w = 3;
		$c = 2;
		$result = $check->check(array('w' => $w, 'c' => $c, 'T' => 'd'));
		$this->assertSame("OK: $left days left|'days left'=$left;$w;$c", $result[1]);
		$this->assertSame(0, $result[0]);
	}

	function testTrialExpired() {
		$this->install_date(strtotime('-45 days'));
		$lic = new op5lic('/trigger/trial/please');
		$sys = new op5sysinfo_mock();
		$check = new Check_Op5license($lic, $sys);
		$result = $check->check(array('w' => 3, 'c' => 2, 'T' => 'd'));
		$this->assertRegExp("/^CRITICAL: /", $result[1]);
		$this->assertSame(2, $result[0]);
	}

	function testPercetageLeft() {
		$allowed = 1000;
		$this->mock(array('monitor.hosts' => $allowed));

		$lic = new op5lic($this->license_file);
		$sys = new op5sysinfo_mock();
		$sys->mon = 10;
		$left = 99;
		$check = new Check_Op5license($lic, $sys);

		$w = 1000;
		$c = 10;
		$result = $check->check(array('w' => $w, 'c' => $c, 'T' => 'p'));
		$this->assertSame("WARNING: $left% hosts left|'% hosts left'=$left%;$w;$c 'hosts allowed'=".$allowed, $result[1]);
		$this->assertSame(1, $result[0]);

		$w = 2000;
		$c = 1000;
		$result = $check->check(array('w' => $w, 'c' => $c, 'T' => 'p'));
		$this->assertSame("CRITICAL: $left% hosts left|'% hosts left'=$left%;$w;$c 'hosts allowed'=".$allowed, $result[1]);
		$this->assertSame(2, $result[0]);

		$w = 50;
		$c = 20;
		$result = $check->check(array('w' => $w, 'c' => $c, 'T' => 'p'));
		$this->assertSame("OK: $left% hosts left|'% hosts left'=$left%;$w;$c 'hosts allowed'=".$allowed, $result[1]);
		$this->assertSame(0, $result[0]);
	}

	function testMissingArgs() {
		$this->_exec($this->command_prefix, array('-w' => '20'), 'UNKNOWN: Missing critical threshold', 3);
		$this->_exec($this->command_prefix, array('-c' => '20'), 'UNKNOWN: Missing warning threshold', 3);
	}

	function testInvalidArgs() {
		$this->_exec($this->command_prefix, array('-w' => '50', '-c' => '30', '-T' => 'd', '-P' => 'm'), 'UNKNOWN: Invalid product name, see --help for explanations to the P parameter', 3);
	}

	function testTrialTimeLeft() {
		// less than 30 days ago which is the limit
		$this->install_date(strtotime('-20 days'));
		$lic = new op5lic('trigger a trial license, please');
		$sys = new op5sysinfo_mock();
		$sys->mon = 10;
		$check = new Check_Op5license($lic, $sys);

		# how many hosts can a trial license have? unlimited!
		$w = 2;
		$c = 1;
		$result = $check->check(array('w' => $w, 'c' => $c, 'T' => 'p'));
		$this->assertSame("OK: Your license is valid", $result[1]);
		$this->assertSame(0, $result[0]);
	}

	function testTrialEnded() {
		$this->install_date(strtotime('-40 days')); // more than 30 days ago which is the limit
		$lic = new op5lic('/invalid-path/4-le_liz3nce..it will trigger a trial license being set');
		$sys = new op5sysinfo_mock();
		$sys->mon = 10;
		$check = new Check_Op5license($lic, $sys);

		# how many hosts can a trial license have? unlimited!
		$w = 2;
		$c = 1;
		$result = $check->check(array('w' => $w, 'c' => $c, 'T' => 'p'));
		$this->assertRegExp("/^CRITICAL: /", $result[1]);
		$this->assertRegExp("/Your license has expired/", $result[1]);
		$this->assertSame(2, $result[0]);
	}
}
